import { app } from '@/app.ts';
import { FileSyncModel } from '@/file-sync/model.ts';
import { logger } from '@/modules/logger.ts';
import { oss, SyncFile, SyncType } from '@/modules/minio.ts';
import { BucketItemStat } from 'minio';
type Result = {
  code: number;
  message?: string;
  data?: any;
  duration?: number;
  error?: string;
};
app
  .route({
    path: 'file-listener',
    key: 'sync-all',
  })
  .define(async (ctx) => {
    const startTime = Date.now();
    const runSync = async () => {
      return new Promise<Result>((resolve, reject) => {
        const dataStream = oss.client.listObjectsV2(oss.bucketName, '', true);
        dataStream.on('data', async (data) => {
          if (data.name?.startsWith?.('private')) return;
          app.call({
            path: 'file-listener',
            key: 'sync-file',
            payload: {
              data: data,
            },
          });
        });
        dataStream.on('end', () => {
          const endTime = Date.now();
          logger.info(`Sync completed in ${endTime - startTime} ms`);
          resolve({ code: 200, message: 'Sync completed successfully', duration: endTime - startTime });
        });
        dataStream.on('error', (err) => {
          console.error('Error during sync:', err);
          ctx.body = { message: 'Sync failed', error: err.message };
          resolve({ code: 500, message: 'Sync failed', error: err.message });
        });
      });
    };
    const res = await runSync();
    if (res.code === 200) {
      ctx.body = res.data;
    }
    ctx.throw(res.code, res.message || 'Sync operation completed');
  })
  .addTo(app);

app
  .route({
    path: 'file-listener',
    key: 'sync-file',
  })
  .define(async (ctx) => {
    const { data } = ctx.query;
    if (!data.name && !data.key) {
      ctx.throw(400, 'No data provided for sync');
    }
    const type: SyncType = data.type || 'sync';
    const name = data.name || data.key;
    logger.info('Syncing file:', name, 'Type:', type);
    let fileSync: FileSyncModel = null;
    if (type === 'delete') {
      const deleteNum = await FileSyncModel.destroy({
        where: { name },
      });
      if (deleteNum) {
        logger.info('File deleted from sync model:', name);
      }
      return;
    }
    const bucketItemStat: BucketItemStat = await oss.statObject(name);

    if (type === 'create' || type === 'update' || type === 'sync') {
      const [fileSync, created] = await FileSyncModel.findOrCreate({
        where: { name },
        defaults: {
          name,
          hash: bucketItemStat.etag || '',
          stat: bucketItemStat || {},
          data: {},
          checkedAt: new Date(),
        },
      });
      if (created) {
        logger.info('File created in sync model:', name);
        return;
      }
      const result = await fileSync.updateStat(bucketItemStat);
      logger.info('File updated in sync model:', name, result);
      ctx.body = result;
      return;
    }

    logger.info('Processing file:', bucketItemStat);
  })
  .addTo(app);
